unit DblListBox;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons;

type
  TDualListBox = class(TListBox)
  private
    { Private declarations }
    FBrother: TDualListBox;
    FAcceptBrotherItems, FAcceptOwnItems: Boolean;
  protected
    { Protected declarations }
    procedure DraggingOver(Sender, Source: TObject; X, Y: Integer; State: TDragState; var Accept: Boolean);
    procedure DragDropped(Sender, Source: TObject; X, Y: Integer);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
  published
    { Published declarations }
    property Brother: TDualListBox read FBrother write FBrother;
    property AcceptBrotherItems: Boolean read FAcceptBrotherItems write FAcceptBrotherItems;
    property AcceptOwnItems: Boolean read FAcceptOwnItems write FAcceptOwnItems;
  end;

  TVisibleButton = (vbRight, vbLeft, vbRightAll, vbLeftAll);
  TVisibleButtons = set of TVisibleButton;

  TMoveOption = (moDblClick, moDragDrop, moWithButton);
  TMoveOptions = set of TMoveOption;

  TDoubleListBox = class(TGroupBox)
  private
    { Private declarations }
    FButtonWidth: Integer;
    FBtnRightOne, FBtnLeftOne, FBtnRightAll, FBtnLeftAll: TSpeedButton;
    FLeftBox, FRightBox: TDualListBox;
    FVisibleButtons: TVisibleButtons;
    FMoveOptions: TMoveOptions;
  protected
    { Protected declarations }
    procedure Resized(Sender: TObject);
    procedure SetButtonWidth(Value: Integer);
    procedure ClickLeft(Sender: TObject);
    procedure ClickRight(Sender: TObject);
    procedure DblClickLeft(Sender: TObject);
    procedure DblClickRight(Sender: TObject);
    procedure LeftOneClick(Sender: TObject);
    procedure RightOneClick(Sender: TObject);
    procedure LeftAllClick(Sender: TObject);
    procedure RightAllClick(Sender: TObject);

    procedure SetItemsLeft(Value: TStrings);
    function GetItemsLeft: TStrings;
    procedure SetItemsRight(Value: TStrings);
    function GetItemsRight: TStrings;
    function GetListColor: TColor;
    procedure SetListColor(Value: TColor);
    procedure SetVisibleButtons(Value: TVisibleButtons);
    procedure SetMoveOptions(Value: TMoveOptions);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    { Published declarations }
    property ButtonWidth: Integer read FButtonWidth write SetButtonWidth;
    property ItemsLeft: TStrings read GetItemsLeft write SetItemsLeft;
    property ItemsRight: TStrings read GetItemsRight write SetItemsRight;
    property ListColor: TColor read GetListColor write SetListColor;
    property VisibleButtons: TVisibleButtons read FVisibleButtons write SetVisibleButtons;
    property MoveOptions: TMoveOptions read FMoveOptions write SetMoveOptions;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Additional', [TDoubleListBox]);
end;

constructor TDualListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  DragMode := dmAutomatic;

  OnDragOver := DraggingOver;
  OnDragDrop := DragDropped;
end;

procedure TDualListBox.DraggingOver(Sender, Source: TObject; X, Y: Integer;
  State: TDragState; var Accept: Boolean);
begin
  Accept := False;

  if NOT (Source is TDualListBox) then
    Exit;

  if (Source = FBrother) and AcceptBrotherItems then
    Accept := True;

  if (Source = Self) and AcceptOwnItems then
    Accept := True;
end;

procedure TDualListBox.DragDropped(Sender, Source: TObject; X, Y: Integer);
var I, AtIndex: Integer;
begin
  if NOT (Source is TListBox) then
    Exit;

  AtIndex := (Sender as TListBox).TopIndex + Y div (Sender as TListBox).ItemHeight;
  I := (Source as TListBox).ItemIndex;
  if I < 0 then
    Exit;

  if AtIndex >= (Sender as TListBox).Items.Count then
    (Sender as TListBox).Items.Add((Source as TListBox).Items[I]) else
    (Sender as TListBox).Items.Insert(AtIndex, (Source as TListBox).Items[I]);

  (Source as TListBox).Items.Delete((Source as TListBox).ItemIndex);
end;





constructor TDoubleListBox.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  FButtonWidth := 22;
  Height := 180;

  FBtnRightOne := TSpeedButton.Create(Self);
  FBtnRightOne.Parent := Self;
  with FBtnRightOne do
    begin
      Caption := '>';
      Width := ButtonWidth;
      Height := ButtonWidth;
      Flat := True;
      Enabled := False;
      OnClick := RightOneClick;
    end;

  FBtnLeftOne := TSpeedButton.Create(Self);
  FBtnLeftOne.Parent := Self;
  with FBtnLeftOne do
    begin
      Parent := Self;
      Caption := '<';
      Width := ButtonWidth;
      Height := ButtonWidth;
      Flat := True;
      Enabled := False;
      OnClick := LeftOneClick;
    end;

  FBtnRightAll := TSpeedButton.Create(Self);
  FBtnRightAll.Parent := Self;
  with FBtnRightAll do
    begin
      Caption := '>>';
      Width := ButtonWidth;
      Height := ButtonWidth;
      Flat := True;
      OnClick := RightAllClick;
    end;

  FBtnLeftAll := TSpeedButton.Create(Self);
  FBtnLeftAll.Parent := Self;
  with FBtnLeftAll do
    begin
      Caption := '<<';
      Width := ButtonWidth;
      Height := ButtonWidth;
      Flat := True;
      OnClick := LeftAllClick;
    end;

  FRightBox := TDualListBox.Create(Self);
  FRightBox.Parent := Self;
  with FRightBox do
    begin
      Align := alRight;
      AcceptBrotherItems := True;
      OnClick := ClickRight;
      OnDblClick := DblClickRight;
    end;

  FLeftBox := TDualListBox.Create(Self);
  FLeftBox.Parent := Self;
  with FLeftBox do
    begin
      Align := alLeft;
      AcceptBrotherItems := True;
      OnClick := ClickLeft;
      OnDblClick := DblClickLeft;
      Brother := FRightBox;
    end;

  FRightBox.Brother := FLeftBox;

  FVisibleButtons := [vbLeft, vbRight, vbLeftAll, vbRightAll];
  FMoveOptions := [moDblClick, moDragDrop, moWithButton]; 
  OnResize := Resized;
end;

destructor TDoubleListBox.Destroy;
begin
  FBtnRightOne.Free;
  FBtnRightAll.Free;
  FBtnLeftOne.Free;
  FBtnLeftAll.Free;

  inherited;
end;

procedure TDoubleListBox.SetItemsLeft(Value: TStrings);
begin
  FLeftBox.Items.Assign(Value);
end;

function TDoubleListBox.GetItemsLeft: TStrings;
begin
  Result := FLeftBox.Items;
end;

procedure TDoubleListBox.SetItemsRight(Value: TStrings);
begin
  FRightBox.Items.Assign(Value);
end;

function TDoubleListBox.GetItemsRight: TStrings;
begin
  Result := FRightBox.Items;
end;

function TDoubleListBox.GetListColor: TColor;
begin
  Result := FRightBox.Color;
end;

procedure TDoubleListBox.SetListColor(Value: TColor);
begin
  FRightBox.Color := Value;
  FLeftBox.Color := Value;
end;

procedure TDoubleListBox.SetVisibleButtons(Value: TVisibleButtons);
begin
  if Value = VisibleButtons then
    Exit;

  FVisibleButtons := Value;
  Resized(Self);
  Invalidate;
end;

procedure TDoubleListBox.SetMoveOptions(Value: TMoveOptions);
begin
  FMoveOptions := Value;

  if moDragDrop in MoveOptions then
    begin
      FLeftBox.DragMode := dmAutomatic;
      FRightBox.DragMode := dmAutomatic;
    end else
    begin
      FLeftBox.DragMode := dmManual;
      FRightBox.DragMode := dmManual;
    end;
end;

procedure TDoubleListBox.SetButtonWidth(Value: Integer);
begin
  FButtonWidth := Value;
  Resized(Self);
end;

procedure TDoubleListBox.Resized(Sender: TObject);
var ListBoxWidth, SpaceWidth: Integer;
begin
  SpaceWidth := 5;
  ListBoxWidth := (ClientWidth - ButtonWidth - SpaceWidth * 2) div 2;

  FLeftBox.Width := ListBoxWidth;
  FRightBox.Width := ListBoxWidth;

  with FBtnRightOne do
    begin
      Left := ListBoxWidth + SpaceWidth;
      Top := SpaceWidth * 3;
      Width := ButtonWidth;
      Height := ButtonWidth;
      Visible := vbRight in VisibleButtons;
    end;

  with FBtnLeftOne do
    begin
      Left := ListBoxWidth + SpaceWidth;
      Top := FBtnRightOne.Top + FBtnRightOne.Height + SpaceWidth;
      Width := ButtonWidth;
      Height := ButtonWidth;
      Visible := vbLeft in VisibleButtons;
    end;

  with FBtnRightAll do
    begin
      Left := ListBoxWidth + SpaceWidth;
      Top := FBtnLeftOne.Top + FBtnLeftOne.Height + SpaceWidth * 2;
      Width := ButtonWidth;
      Height := ButtonWidth;
      Visible := vbRightAll in VisibleButtons;
    end;

  with FBtnLeftAll do
    begin
      Left := ListBoxWidth + SpaceWidth;
      Top := FBtnRightAll.Top + FBtnRightAll.Height + SpaceWidth;
      Width := ButtonWidth;
      Height := ButtonWidth;
      Visible := vbLeftAll in VisibleButtons;
    end;
end;

procedure TDoubleListBox.ClickLeft(Sender: TObject);
var I: Integer;
begin
  I := FLeftBox.ItemIndex;
  FBtnRightOne.Enabled := I >= 0;
end;

procedure TDoubleListBox.ClickRight(Sender: TObject);
var I: Integer;
begin
  I := FRightBox.ItemIndex;
  FBtnLeftOne.Enabled := I >= 0;
end;

procedure TDoubleListBox.DblClickLeft(Sender: TObject);
begin
  if NOT (moDblClick in MoveOptions) then
    Exit;

  RightOneClick(NIL);
end;

procedure TDoubleListBox.DblClickRight(Sender: TObject);
begin
  if NOT (moDblClick in MoveOptions) then
    Exit;

  LeftOneClick(NIL);
end;

procedure TDoubleListBox.LeftOneClick(Sender: TObject);
var I: Integer;
  S: String;
begin
  if (NOT (moWithButton in MoveOptions)) and (Sender <> NIL) then
    Exit;

  I := FRightBox.ItemIndex;
  if I >= 0 then
    begin
      S := FRightBox.Items[I];
      FRightBox.Items.Delete(I);
      FLeftBox.Items.Add(S);
    end;

  ClickRight(Self);
end;

procedure TDoubleListBox.RightOneClick(Sender: TObject);
var I: Integer;
  S: String;
begin
  if (NOT (moWithButton in MoveOptions)) and (Sender <> NIL) then
    Exit;

  I := FLeftBox.ItemIndex;
  if I >= 0 then
    begin
      S := FLeftBox.Items[I];
      FLeftBox.Items.Delete(I);
      FRightBox.Items.Add(S);
    end;

  ClickLeft(Self);
end;

procedure TDoubleListBox.LeftAllClick(Sender: TObject);
begin
  FLeftBox.Items.AddStrings(FRightBox.Items);
  FRightBox.Items.Clear;
end;

procedure TDoubleListBox.RightAllClick(Sender: TObject);
begin
  FRightBox.Items.AddStrings(FLeftBox.Items);
  FLeftBox.Items.Clear;
end;


end.


